#ifndef RTC_h
#define RTC_h

#if defined(ARDUINO) && (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#endif

#define	RTC_UNDEFINED	0	
#define	RTC_DS3231		1	
#define	RTC_DS1302		2	

#define	RTC_I2C			1	
#define	RTC_SPI			2	
#define	RTC_SI3			3	


class RTC{
	public:
		void  begin  (uint8_t,uint8_t=SS,uint8_t=SCK,uint8_t=MOSI);	
		void  period (uint8_t);										
		char* gettime(char*);										
		void  gettime();											
		void  settime(int,int=-1,int=-1,int=-1,int=-1,int=-1,int=-1);

		uint8_t	seconds					=	0;						//		0-59
		uint8_t	minutes					=	0;						//		0-59
		uint8_t	hours					=	12;						//		1-12
		uint8_t	Hours					=	0;						//		0-23
		uint8_t	midday					=	0;						//		0-1		(0-am, 1-pm)
		uint8_t	day						=	1;						//		1-31
		uint8_t	weekday					=	0;						//		0-6	
		uint8_t	month					=	1;						//		1-12
		uint8_t	year					=	15;						//		0-99	
		
	private:
		uint8_t	RTC_uint_MODULE			=	0;			
		uint8_t	RTC_array_BUS[4][8]		=	{					
			/*	RTC_UNDEFINED	0	*/		{0},					
			/*	RTC_DS3231		1	*/		{RTC_I2C,  100, 0x68, 0, 0, 0, 0, 0},
			/*	RTC_DS1302		2	*/		{RTC_SI3,   10,    1, 1, 0, 1, 1,-1},
			/*	RTC_DS1307		3	*/		{RTC_I2C,  100, 0x68, 0, 0, 0, 0, 0},
											};
	   uint16_t	RTC_array_STR[4][11]	=	{						
			/*	RTC_UNDEFINED	0	*/		{0},					
			/*	RTC_DS3231		1	*/		{3,0x02,0b0010000000000000,0x0E,0b1010001010101010,0x0F,0b1000000011001010},
			/*	RTC_DS1302		2	*/		{3,0x8F,0b1000000000000000,0x81,0b1000000000000000,0x85,0b1000000000000000},
			/*	RTC_DS1307		3	*/		{3,0x00,0b1000000000000000,0x02,0b0010000000000000,0x07,0b0000001100001010},
											};
		uint8_t	RTC_array_REG[4][7]		=	{					
			/*	RTC_UNDEFINED	0	*/		{0},				
			/*	RTC_DS3231		1	*/		{0x00,0x01,0x02,0x04,0x05,0x06,0x03},
			/*	RTC_DS1302		2	*/		{0x81,0x83,0x85,0x87,0x89,0x8D,0x8B},
			/*	RTC_DS1307		3	*/		{0x00,0x01,0x02,0x04,0x05,0x06,0x03},
											};
		uint8_t	RTC_ignor_BIT[4][7]		=	{						
			/*	RTC_UNDEFINED	0	*/		{0},					
			/*	RTC_DS3231		1	*/		{0x7F,0x7F,0x3F,0x3F,0x1F,0xFF,0x07},
			/*	RTC_DS1302		2	*/		{0x7F,0x7F,0x3F,0x3F,0x1F,0xFF,0x07},
			/*	RTC_DS1307		3	*/		{0x7F,0x7F,0x3F,0x3F,0x1F,0xFF,0x07},
											};

	/**	================================================================================================================================= **/

		void	RTC_func_START();									
		void	RTC_func_READ_TIME();								
		void	RTC_func_WRITE_TIME(int,int,int,int,int,int,int);	
		uint8_t	RTC_func_READ_TIME_INDEX(uint8_t);					
		void	RTC_func_WRITE_TIME_INDEX(uint8_t, uint8_t);		
		uint8_t	RTC_func_READ_REG(uint8_t);							
		void	RTC_func_WRITE_REG(uint8_t, uint8_t);				
		uint8_t	RTC_func_CONVER_CODE_NUM(uint8_t);					
		uint8_t	RTC_func_CONVER_NUM_CODE(uint8_t);					
		void	RTC_func_SET_MORE_TIME();							
		void	RTC_func_CALC_TIME();								
		void	RTC_func_FILL_RESULTS(uint8_t,uint8_t);				

		char	RTC_char_RETURN[50];								
		char	RTC_uint_RETURN			=	0;						
		char	RTC_char_MIDDAY[4][3]	=	{"am","pm","AM","PM"};
		char	RTC_char_DayMon[19][4]	=	{"Sun","Mon","Tue","Wed","Thu","Fri","Sat","Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
		uint8_t	RTC_buff_TIMEREG[7];								
		uint8_t	RTC_uint_CENTURY		=	21;						
	   uint16_t	RTC_uint_PERIOD			=	0;						
	   uint32_t	RTC_uint_REQUEST		=	0;						

	/**	I2C, SPI, SI3 **/
	//										
		uint8_t	SI3_line_CLK			=	SCK;					
		uint8_t	SI3_line_DAT			=	MOSI;					
		uint8_t	SI3_line_RES			=	SS;						
		uint8_t	SPI_line_SS				=	SS;						
	//		
	   uint16_t	SPI_mass_STATUS[0x06]	=	{4000,1000,1,0,1,0};	
	   uint16_t	I2C_mass_STATUS[0x04]	=	{ 100,1000,1,0x78 };	
	   uint16_t	SI3_mass_STATUS[0x06]	=	{  10,1000,1,0,1,1};	
	//		
		void	I2C_func_begin			();							
		void	SPI_func_begin			();							
		void	SI3_func_begin			(uint8_t, uint8_t);			
	//			
		void	I2C_func_START			();							
		void	I2C_func_RESTART		();							
		void	I2C_func_STOP			();							
		void	I2C_func_SEND_ID		(uint8_t, bool);			
	//			
		void	I2C_func_WRITE_BYTE		(uint8_t);					
		uint8_t	I2C_func_READ_BYTE		(bool);						
		uint8_t	SPI_func_WRITEREAD_BYTE	(uint8_t);					
		void	SI3_func_WRITE_BYTE		(uint8_t);					
		uint8_t	SI3_func_READ_BYTE		(bool);						
	//			
		void	SI3_func_WRITE_BITS		(uint32_t, uint8_t);		
	   uint32_t	SI3_func_READ_BITS		(uint8_t, bool = false);	
																	
};

#endif